using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DatePair
{
    public partial class Form1 : Form
    {
        const ulong maxDateValue = 6071500800;

        ulong yearOffset = 0;
        ulong dayOffset = 0;
        ulong hourOffset = 0;
        ulong minuteOffset = 0;

        public Form1() {
            InitializeComponent();

            this.dateTimePicker1.Value = new DateTime(1900, 10, 1, 1, 1, 1);
            this.dateTimePicker2.Value = new DateTime(1900, 10, 2, 1, 1, 1);


            yearOffset = 366 * 24 * 60 * 60;
            dayOffset = 24 * 60 * 60;
            hourOffset = 60 * 60;
            minuteOffset = 60;
        }


        private ulong PairDates(DateTime d1, DateTime d2) {

            ulong x = ConvertDate(d1);
            ulong y = ConvertDate(d2);

            ulong n = maxDateValue;
            ulong m = n - x + 2;
            ulong pos = 0;

            if (m > n) {
                pos = (ulong)(maxDateValue - y + 1);
            }
            else {
                pos = (ulong)((n - m + 1) * (n + m)) / 2 + (maxDateValue - y + 1);
            }

            return pos;
        }

        private ulong ConvertDate(DateTime d) {
            return  (ulong)(d.Year - 1900) * yearOffset + (ulong)d.DayOfYear * dayOffset + (ulong)d.Hour * hourOffset + (ulong)d.Minute * minuteOffset + (ulong)d.Second;
        }

        private void UnpairDates(ulong pairDate, out DateTime d1, out DateTime d2)
        {
            bool search = true;
            ulong sum = 0;
            ulong total = 0;
            ulong stepen = 1;

            while (search) {           

                stepen = stepen << 1;                
                total += stepen;
                sum = Sum(maxDateValue, maxDateValue - total + 1);

                if (sum >= pairDate)
                {                    
                    total -= stepen;

                    stepen = 1;                 
                    total += stepen;

                    sum = Sum(maxDateValue, maxDateValue - total + 1);                    

                    if (sum >= pairDate) {
                        break;
                    }                  
                }                
            }

            ulong ud1 = total;
            sum = Sum(maxDateValue, maxDateValue - total + 2);
            ulong ud2 = maxDateValue - (pairDate - sum) + 1;


            d1 = GetDate(ud1);
            d2 = GetDate(ud2);
        }


        private DateTime GetDate(ulong pos) {
            
            ulong year = pos / yearOffset;
            ulong day = (pos - year * yearOffset) / dayOffset;

            ulong hour = (pos - year * yearOffset - day * dayOffset) / hourOffset;
            ulong minute = (pos - year * yearOffset - day * dayOffset - hour * hourOffset) / minuteOffset;
            ulong second = pos - year * yearOffset - day * dayOffset - hour * hourOffset - minute * minuteOffset;            

            DateTime d = new DateTime(1900 + (int)year, 1, 1);
            DateTime d2 = d.AddDays(day - 1);
            

            return new DateTime(d2.Year, d2.Month, d2.Day, (int)hour, (int)minute, (int)second);
        }

        private ulong Sum(ulong n, ulong m)
        {
            return ((n - m + 1) * (n + m)) / 2;
        }

        private void button1_Click(object sender, EventArgs e) {
                   
            ulong res = PairDates(dateTimePicker1.Value, dateTimePicker2.Value);

            DateTime d1;
            DateTime d2;
            
            UnpairDates(res, out d1, out d2);

            dateTimePicker3.Value = d1;
            dateTimePicker4.Value = d2;
            
            textBox1.Text = res.ToString();
            
        }
    }
}